@@ -1,3 +1,4 @@
+from ._version import __version__
 from .binning import BinningProcess
 from .binning import ContinuousOptimalBinning
 from .binning import MDLP
@@ -5,13 +6,15 @@ from .binning import MulticlassOptimalBinning
 from .binning import OptimalBinning
 from .binning.distributed import BinningProcessSketch
 from .binning.distributed import OptimalBinningSketch
+from .binning.multidimensional import OptimalBinning2D
 from .binning.piecewise import ContinuousOptimalPWBinning
 from .binning.piecewise import OptimalPWBinning
 from .binning.uncertainty import SBOptimalBinning
 from .scorecard import Scorecard
 
 
-__all__ = ['BinningProcess',
+__all__ = ['__version__',
+           'BinningProcess',
            'BinningProcessSketch',
            'ContinuousOptimalBinning',
            'ContinuousOptimalPWBinning',
@@ -19,6 +22,7 @@ __all__ = ['BinningProcess',
            'MulticlassOptimalBinning',
            'OptimalBinning',
            'OptimalBinningSketch',
+           'OptimalBinning2D',
            'OptimalPWBinning',
            'SBOptimalBinning',
            'Scorecard']
