@@ -12,6 +12,7 @@ import numpy as np
 
 from sklearn.utils import check_array
 
+from ..information import solver_statistics
 from ..logging import Logger
 from .auto_monotonic import auto_monotonic
 from .auto_monotonic import peak_valley_trend_change_heuristic
@@ -21,6 +22,7 @@ from .binning_statistics import bin_categorical
 from .binning_statistics import bin_info
 from .binning_statistics import BinningTable
 from .binning_statistics import target_info_samples
+from .binning_statistics import target_info_special
 from .cp import BinningCP
 from .ls import BinningLS
 from .mip import BinningMIP
@@ -30,6 +32,9 @@ from .preprocessing import split_data
 from .transformations import transform_binary_target
 
 
+logger = Logger(__name__).logger
+
+
 def _check_parameters(name, dtype, prebinning_method, solver, divergence,
                       max_n_prebins, min_prebin_size, min_n_bins, max_n_bins,
                       min_bin_size, max_bin_size, min_bin_n_nonevent,
@@ -211,8 +216,13 @@ def _check_parameters(name, dtype, prebinning_method, solver, divergence,
                                                    len(user_splits_fixed)))
 
     if special_codes is not None:
-        if not isinstance(special_codes, (np.ndarray, list)):
-            raise TypeError("special_codes must be a list or numpy.ndarray.")
+        if not isinstance(special_codes, (np.ndarray, list, dict)):
+            raise TypeError("special_codes must be a dit, list or "
+                            "numpy.ndarray.")
+
+        if isinstance(special_codes, dict) and not len(special_codes):
+            raise ValueError("special_codes empty. special_codes dict must "
+                             "contain at least one special.")
 
     if split_digits is not None:
         if (not isinstance(split_digits, numbers.Integral) or
@@ -371,7 +381,7 @@ class OptimalBinning(BaseOptimalBinning):
 
         .. versionadded:: 0.5.0
 
-    special_codes : array-like or None, optional (default=None)
+    special_codes : array-like, dict or None, optional (default=None)
         List of special codes. Use special codes to specify the data values
         that must be treated separately.
 
@@ -494,12 +504,9 @@ class OptimalBinning(BaseOptimalBinning):
         self._time_preprocessing = None
         self._time_prebinning = None
         self._time_solver = None
+        self._time_optimizer = None
         self._time_postprocessing = None
 
-        # logger
-        self._class_logger = Logger(__name__)
-        self._logger = self._class_logger.logger
-
         self._is_fitted = False
 
     def fit(self, x, y, sample_weight=None, check_input=False):
@@ -652,7 +659,7 @@ class OptimalBinning(BaseOptimalBinning):
         binning_type = self.__class__.__name__.lower()
 
         if self._optimizer is not None:
-            solver = self._optimizer.solver_
+            solver = self._optimizer
             time_solver = self._time_solver
         else:
             solver = None
@@ -664,6 +671,7 @@ class OptimalBinning(BaseOptimalBinning):
                                   self._status, self.solver, solver,
                                   self._time_total, self._time_preprocessing,
                                   self._time_prebinning, time_solver,
+                                  self._time_optimizer,
                                   self._time_postprocessing, self._n_prebins,
                                   self._n_refinements, dict_user_options)
 
@@ -671,20 +679,20 @@ class OptimalBinning(BaseOptimalBinning):
         time_init = time.perf_counter()
 
         if self.verbose:
-            self._logger.info("Optimal binning started.")
-            self._logger.info("Options: check parameters.")
+            logger.info("Optimal binning started.")
+            logger.info("Options: check parameters.")
 
         _check_parameters(**self.get_params())
 
         # Pre-processing
         if self.verbose:
-            self._logger.info("Pre-processing started.")
+            logger.info("Pre-processing started.")
 
         self._n_samples = len(x)
 
         if self.verbose:
-            self._logger.info("Pre-processing: number of samples: {}"
-                              .format(self._n_samples))
+            logger.info("Pre-processing: number of samples: {}"
+                        .format(self._n_samples))
 
         time_preprocessing = time.perf_counter()
 
@@ -702,40 +710,40 @@ class OptimalBinning(BaseOptimalBinning):
             n_missing = len(x_missing)
             n_special = len(x_special)
 
-            self._logger.info("Pre-processing: number of clean samples: {}"
-                              .format(n_clean))
+            logger.info("Pre-processing: number of clean samples: {}"
+                        .format(n_clean))
 
-            self._logger.info("Pre-processing: number of missing samples: {}"
-                              .format(n_missing))
+            logger.info("Pre-processing: number of missing samples: {}"
+                        .format(n_missing))
 
-            self._logger.info("Pre-processing: number of special samples: {}"
-                              .format(n_special))
+            logger.info("Pre-processing: number of special samples: {}"
+                        .format(n_special))
 
             if self.outlier_detector is not None:
                 n_outlier = self._n_samples-(n_clean + n_missing + n_special)
-                self._logger.info("Pre-processing: number of outlier samples: "
-                                  "{}".format(n_outlier))
+                logger.info("Pre-processing: number of outlier samples: "
+                            "{}".format(n_outlier))
 
             if self.dtype == "categorical":
                 n_categories = len(categories)
                 n_categories_others = len(cat_others)
                 n_others = len(y_others)
 
-                self._logger.info("Pre-processing: number of others samples: "
-                                  "{}".format(n_others))
+                logger.info("Pre-processing: number of others samples: "
+                            "{}".format(n_others))
 
-                self._logger.info("Pre-processing: number of categories: {}"
-                                  .format(n_categories))
+                logger.info("Pre-processing: number of categories: {}"
+                            .format(n_categories))
 
-                self._logger.info("Pre-processing: number of categories "
-                                  "others: {}".format(n_categories_others))
+                logger.info("Pre-processing: number of categories "
+                            "others: {}".format(n_categories_others))
 
-            self._logger.info("Pre-processing terminated. Time: {:.4f}s"
-                              .format(self._time_preprocessing))
+            logger.info("Pre-processing terminated. Time: {:.4f}s"
+                        .format(self._time_preprocessing))
 
         # Pre-binning
         if self.verbose:
-            self._logger.info("Pre-binning started.")
+            logger.info("Pre-binning started.")
 
         time_prebinning = time.perf_counter()
 
@@ -743,8 +751,8 @@ class OptimalBinning(BaseOptimalBinning):
             n_splits = len(self.user_splits)
 
             if self.verbose:
-                self._logger.info("Pre-binning: user splits supplied: {}"
-                                  .format(n_splits))
+                logger.info("Pre-binning: user splits supplied: {}"
+                            .format(n_splits))
 
             if not n_splits:
                 splits = self.user_splits
@@ -772,11 +780,12 @@ class OptimalBinning(BaseOptimalBinning):
                         self.user_splits_fixed)[sorted_idx]
 
                 splits, n_nonevent, n_event = self._prebinning_refinement(
-                    user_splits, x_clean, y_clean, y_missing, y_special,
-                    y_others, sw_clean, sw_missing, sw_special, sw_others)
+                    user_splits, x_clean, y_clean, y_missing, x_special,
+                    y_special, y_others, sw_clean, sw_missing, sw_special,
+                    sw_others)
         else:
             splits, n_nonevent, n_event = self._fit_prebinning(
-                x_clean, y_clean, y_missing, y_special, y_others,
+                x_clean, y_clean, y_missing, x_special, y_special, y_others,
                 self.class_weight, sw_clean, sw_missing, sw_special, sw_others)
 
         self._n_prebins = len(n_nonevent)
@@ -787,21 +796,21 @@ class OptimalBinning(BaseOptimalBinning):
         self._time_prebinning = time.perf_counter() - time_prebinning
 
         if self.verbose:
-            self._logger.info("Pre-binning: number of prebins: {}"
-                              .format(self._n_prebins))
-            self._logger.info("Pre-binning: number of refinements: {}"
-                              .format(self._n_refinements))
+            logger.info("Pre-binning: number of prebins: {}"
+                        .format(self._n_prebins))
+            logger.info("Pre-binning: number of refinements: {}"
+                        .format(self._n_refinements))
 
-            self._logger.info("Pre-binning terminated. Time: {:.4f}s"
-                              .format(self._time_prebinning))
+            logger.info("Pre-binning terminated. Time: {:.4f}s"
+                        .format(self._time_prebinning))
 
         # Optimization
         self._fit_optimizer(splits, n_nonevent, n_event)
 
         # Post-processing
         if self.verbose:
-            self._logger.info("Post-processing started.")
-            self._logger.info("Post-processing: compute binning information.")
+            logger.info("Post-processing started.")
+            logger.info("Post-processing: compute binning information.")
 
         time_postprocessing = time.perf_counter()
 
@@ -824,30 +833,28 @@ class OptimalBinning(BaseOptimalBinning):
             max_x = None
 
         self._binning_table = BinningTable(
-            self.name, self.dtype, self._splits_optimal, self._n_nonevent,
-            self._n_event, min_x, max_x, self._categories,
+            self.name, self.dtype, self.special_codes, self._splits_optimal,
+            self._n_nonevent, self._n_event, min_x, max_x, self._categories,
             self._cat_others, self.user_splits)
 
         self._time_postprocessing = time.perf_counter() - time_postprocessing
 
         if self.verbose:
-            self._logger.info("Post-processing terminated. Time: {:.4f}s"
-                              .format(self._time_postprocessing))
+            logger.info("Post-processing terminated. Time: {:.4f}s"
+                        .format(self._time_postprocessing))
 
         self._time_total = time.perf_counter() - time_init
 
         if self.verbose:
-            self._logger.info("Optimal binning terminated. Status: {}. "
-                              "Time: {:.4f}s"
-                              .format(self._status, self._time_total))
+            logger.info("Optimal binning terminated. Status: {}. Time: {:.4f}s"
+                        .format(self._status, self._time_total))
 
         # Completed successfully
-        self._class_logger.close()
         self._is_fitted = True
 
         return self
 
-    def _fit_prebinning(self, x, y, y_missing, y_special, y_others,
+    def _fit_prebinning(self, x, y, y_missing, x_special, y_special, y_others,
                         class_weight=None, sw_clean=None, sw_missing=None,
                         sw_special=None, sw_others=None):
 
@@ -862,12 +869,13 @@ class OptimalBinning(BaseOptimalBinning):
                                 ).fit(x, y, sw_clean)
 
         return self._prebinning_refinement(prebinning.splits, x, y, y_missing,
-                                           y_special, y_others, sw_clean,
-                                           sw_missing, sw_special, sw_others)
+                                           x_special, y_special, y_others,
+                                           sw_clean, sw_missing, sw_special,
+                                           sw_others)
 
     def _fit_optimizer(self, splits, n_nonevent, n_event):
         if self.verbose:
-            self._logger.info("Optimizer started.")
+            logger.info("Optimizer started.")
 
         time_init = time.perf_counter()
 
@@ -877,11 +885,11 @@ class OptimalBinning(BaseOptimalBinning):
             self._solution = np.zeros(len(splits)).astype(bool)
 
             if self.verbose:
-                self._logger.warning("Optimizer: {} bins after pre-binning."
-                                     .format(len(n_nonevent)))
-                self._logger.warning("Optimizer: solver not run.")
+                logger.warning("Optimizer: {} bins after pre-binning."
+                               .format(len(n_nonevent)))
+                logger.warning("Optimizer: solver not run.")
 
-                self._logger.info("Optimizer terminated. Time: 0s")
+                logger.info("Optimizer terminated. Time: 0s")
             return
 
         # Min/max number of bins
@@ -932,8 +940,8 @@ class OptimalBinning(BaseOptimalBinning):
                             event_rate, monotonic)
 
                 if self.verbose:
-                    self._logger.info("Optimizer: classifier predicts {} "
-                                      "monotonic trend.".format(monotonic))
+                    logger.info("Optimizer: classifier predicts {} "
+                                "monotonic trend.".format(monotonic))
             else:
                 monotonic = self.monotonic_trend
 
@@ -943,8 +951,8 @@ class OptimalBinning(BaseOptimalBinning):
                         event_rate, monotonic)
 
                     if self.verbose:
-                        self._logger.info("Optimizer: trend change position "
-                                          "{}.".format(trend_change))
+                        logger.info("Optimizer: trend change position {}."
+                                    .format(trend_change))
 
         else:
             monotonic = self.monotonic_trend
@@ -953,11 +961,10 @@ class OptimalBinning(BaseOptimalBinning):
 
         if self.verbose:
             if monotonic is None:
-                self._logger.info(
-                    "Optimizer: monotonic trend not set.")
+                logger.info("Optimizer: monotonic trend not set.")
             else:
-                self._logger.info("Optimizer: monotonic trend set to "
-                                  "{}.".format(monotonic))
+                logger.info("Optimizer: monotonic trend set to {}."
+                            .format(monotonic))
 
         if self.solver == "cp":
             optimizer = BinningCP(monotonic, self.min_n_bins, self.max_n_bins,
@@ -986,19 +993,20 @@ class OptimalBinning(BaseOptimalBinning):
                                   self.user_splits_fixed, self.time_limit)
 
         if self.verbose:
-            self._logger.info("Optimizer: build model...")
+            logger.info("Optimizer: build model...")
 
         optimizer.build_model(self.divergence, n_nonevent, n_event,
                               trend_change)
 
         if self.verbose:
-            self._logger.info("Optimizer: solve...")
+            logger.info("Optimizer: solve...")
 
         status, solution = optimizer.solve()
 
         self._solution = solution
 
-        self._optimizer = optimizer
+        self._optimizer, self._time_optimizer = solver_statistics(
+            self.solver, optimizer.solver_)
         self._status = status
 
         if self.dtype == "categorical" and self.user_splits is not None:
@@ -1009,28 +1017,26 @@ class OptimalBinning(BaseOptimalBinning):
         self._time_solver = time.perf_counter() - time_init
 
         if self.verbose:
-            self._logger.info("Optimizer terminated. Time: {:.4f}s"
-                              .format(self._time_solver))
+            logger.info("Optimizer terminated. Time: {:.4f}s"
+                        .format(self._time_solver))
 
     def _prebinning_refinement(self, splits_prebinning, x, y, y_missing,
-                               y_special, y_others, sw_clean, sw_missing,
-                               sw_special, sw_others):
+                               x_special, y_special, y_others, sw_clean,
+                               sw_missing, sw_special, sw_others):
         y0 = (y == 0)
         y1 = ~y0
 
         # Compute n_nonevent and n_event for special, missing and others.
-        special_target_info = target_info_samples(y_special, sw_special)
-        self._n_nonevent_special = special_target_info[0]
-        self._n_event_special = special_target_info[1]
+        self._n_nonevent_special, self._n_event_special = target_info_special(
+            self.special_codes, x_special, y_special, sw_special)
 
-        missing_target_info = target_info_samples(y_missing, sw_missing)
-        self._n_nonevent_missing = missing_target_info[0]
-        self._n_event_missing = missing_target_info[1]
+        self._n_nonevent_missing, self._n_event_missing = target_info_samples(
+            y_missing, sw_missing)
 
         if len(y_others):
-            others_target_info = target_info_samples(y_others, sw_others)
-            self._n_nonevent_cat_others = others_target_info[0]
-            self._n_event_cat_others = others_target_info[1]
+            (self._n_nonevent_cat_others,
+             self._n_event_cat_others) = target_info_samples(
+             y_others, sw_others)
 
         n_splits = len(splits_prebinning)
 
@@ -1099,8 +1105,8 @@ class OptimalBinning(BaseOptimalBinning):
                 splits = splits_prebinning[~mask_splits]
 
                 if self.verbose:
-                    self._logger.info("Pre-binning: number prebins removed: {}"
-                                      .format(np.count_nonzero(mask_remove)))
+                    logger.info("Pre-binning: number prebins removed: {}"
+                                .format(np.count_nonzero(mask_remove)))
 
                 [splits_prebinning, n_nonevent,
                  n_event] = self._compute_prebins(splits, x, y0, y1, sw)
