@@ -31,6 +31,9 @@ from .piecewise.binning import OptimalPWBinning
 from .piecewise.continuous_binning import ContinuousOptimalPWBinning
 
 
+logger = Logger(__name__).logger
+
+
 _METRICS = {
     "binary": {
         "metrics": ["iv", "js", "gini", "quality_score"],
@@ -272,8 +275,13 @@ def _check_parameters(variable_names, max_n_prebins, min_prebin_size,
                             "strings.")
 
     if special_codes is not None:
-        if not isinstance(special_codes, (np.ndarray, list)):
-            raise TypeError("special_codes must be a list or numpy.ndarray.")
+        if not isinstance(special_codes, (np.ndarray, list, dict)):
+            raise TypeError("special_codes must be a dit, list or "
+                            "numpy.ndarray.")
+
+        if isinstance(special_codes, dict) and not len(special_codes):
+            raise ValueError("special_codes empty. special_codes dict must "
+                             "contain at least one special.")
 
     if split_digits is not None:
         if (not isinstance(split_digits, numbers.Integral) or
@@ -581,10 +589,6 @@ class BinningProcess(Base, BaseEstimator, BaseBinningProcess):
         # timing
         self._time_total = None
 
-        # logger
-        self._class_logger = Logger(__name__)
-        self._logger = self._class_logger.logger
-
         self._is_fitted = False
 
     def fit(self, X, y, sample_weight=None, check_input=False):
@@ -1036,8 +1040,8 @@ class BinningProcess(Base, BaseEstimator, BaseBinningProcess):
         time_init = time.perf_counter()
 
         if self.verbose:
-            self._logger.info("Binning process started.")
-            self._logger.info("Options: check parameters.")
+            logger.info("Binning process started.")
+            logger.info("Options: check parameters.")
 
         _check_parameters(**self.get_params())
 
@@ -1078,24 +1082,24 @@ class BinningProcess(Base, BaseEstimator, BaseBinningProcess):
                              "length of variable_names.")
 
         if self.verbose:
-            self._logger.info("Dataset: number of samples: {}."
-                              .format(self._n_samples))
+            logger.info("Dataset: number of samples: {}."
+                        .format(self._n_samples))
 
-            self._logger.info("Dataset: number of variables: {}."
-                              .format(self._n_variables))
+            logger.info("Dataset: number of variables: {}."
+                        .format(self._n_variables))
 
         # Number of jobs
         n_jobs = effective_n_jobs(self.n_jobs)
 
         if self.verbose:
-            self._logger.info("Options: number of jobs (cores): {}."
-                              .format(n_jobs))
+            logger.info("Options: number of jobs (cores): {}."
+                        .format(n_jobs))
 
         if n_jobs == 1:
             for i, name in enumerate(self.variable_names):
                 if self.verbose:
-                    self._logger.info("Binning variable ({} / {}): {}."
-                                      .format(i, self._n_variables, name))
+                    logger.info("Binning variable ({} / {}): {}."
+                                .format(i, self._n_variables, name))
 
                 if isinstance(X, np.ndarray):
                     dtype, optb = _fit_variable(
@@ -1151,7 +1155,7 @@ class BinningProcess(Base, BaseEstimator, BaseBinningProcess):
                 self._binned_variables.update(bv)
 
         if self.verbose:
-            self._logger.info("Binning process variable selection...")
+            logger.info("Binning process variable selection...")
 
         # Compute binning statistics and decide whether a variable is selected
         self._binning_selection_criteria()
@@ -1159,11 +1163,10 @@ class BinningProcess(Base, BaseEstimator, BaseBinningProcess):
         self._time_total = time.perf_counter() - time_init
 
         if self.verbose:
-            self._logger.info("Binning process terminated. Time: {:.4f}s"
-                              .format(self._time_total))
+            logger.info("Binning process terminated. Time: {:.4f}s"
+                        .format(self._time_total))
 
         # Completed successfully
-        self._class_logger.close()
         self._is_fitted = True
 
         return self
@@ -1172,8 +1175,8 @@ class BinningProcess(Base, BaseEstimator, BaseBinningProcess):
         time_init = time.perf_counter()
 
         if self.verbose:
-            self._logger.info("Binning process started.")
-            self._logger.info("Options: check parameters.")
+            logger.info("Binning process started.")
+            logger.info("Options: check parameters.")
 
         _check_parameters(**self.get_params())
 
@@ -1211,11 +1214,11 @@ class BinningProcess(Base, BaseEstimator, BaseBinningProcess):
         self._n_variables = len(self.variable_names)
 
         if self.verbose:
-            self._logger.info("Dataset: number of samples: {}."
-                              .format(self._n_samples))
+            logger.info("Dataset: number of samples: {}."
+                        .format(self._n_samples))
 
-            self._logger.info("Dataset: number of variables: {}."
-                              .format(self._n_variables))
+            logger.info("Dataset: number of variables: {}."
+                        .format(self._n_variables))
 
         for name in self.variable_names:
             x = _read_column(input_path, extension, name, **kwargs)
@@ -1231,7 +1234,7 @@ class BinningProcess(Base, BaseEstimator, BaseBinningProcess):
             self._binned_variables[name] = optb
 
         if self.verbose:
-            self._logger.info("Binning process variable selection...")
+            logger.info("Binning process variable selection...")
 
         # Compute binning statistics and decide whether a variable is selected
         self._binning_selection_criteria()
@@ -1239,11 +1242,10 @@ class BinningProcess(Base, BaseEstimator, BaseBinningProcess):
         self._time_total = time.perf_counter() - time_init
 
         if self.verbose:
-            self._logger.info("Binning process terminated. Time: {:.4f}s"
-                              .format(self._time_total))
+            logger.info("Binning process terminated. Time: {:.4f}s"
+                        .format(self._time_total))
 
         # Completed successfully
-        self._class_logger.close()
         self._is_fitted = True
 
         return self
@@ -1252,8 +1254,8 @@ class BinningProcess(Base, BaseEstimator, BaseBinningProcess):
         time_init = time.perf_counter()
 
         if self.verbose:
-            self._logger.info("Binning process started.")
-            self._logger.info("Options: check parameters.")
+            logger.info("Binning process started.")
+            logger.info("Options: check parameters.")
 
         _check_parameters(**self.get_params())
 
@@ -1319,11 +1321,10 @@ class BinningProcess(Base, BaseEstimator, BaseBinningProcess):
         self._time_total = time.perf_counter() - time_init
 
         if self.verbose:
-            self._logger.info("Binning process terminated. Time: {:.4f}s"
-                              .format(self._time_total))
+            logger.info("Binning process terminated. Time: {:.4f}s"
+                        .format(self._time_total))
 
         # Completed successfully
-        self._class_logger.close()
         self._is_fitted = True
 
         return self
